require 'faraday'
require 'json'

module PixivApi
  class Authentication
    # Authentication provides password authentication for development.
    class InvalidAuthError < StandardError; end

    def initialize(id:, password:)
      connection = Faraday.new(default_options) do |conn|
        # Set default middleware
        conn.request :multipart
        conn.request :url_encoded
        conn.adapter Faraday.default_adapter
      end

      @response = connection.post(
        URI(configuration[:token_url]).path,
        client_id: configuration[:client_id],
        client_secret: configuration[:client_secret],
        grant_type: 'password',
        username: id,
        password: password
      )

      raise InvalidAuthError, body['errors'].to_s unless success?
    end

    def body
      @body ||= JSON.parse(@response.body).freeze
    end

    def to_client
      PixivApi::Client.new(
        access_token: body['access_token'],
        refresh_token: body['refresh_token'],
        expires_at: Time.now.to_i + body['expires_in']
      )
    end

    def success?
      @response.success?
    end

    private

    def configuration
      PixivApi.configuration.tap(&:require_keys!)
    end

    def default_options
      {
        headers: {
          'Host' => URI(configuration[:token_url]).host,
        },
        url: configuration[:site],
      }
    end
  end
end
