module PixivApi
  class Response
    class Work < Identity
      def self.from_response(response, *args)
        attributes = args.extract_options!

        base = case attributes['type']
               when 'illustration'
                 Illustration
               when 'manga'
                 Manga
               when 'ugoira'
                 Ugoira
               else
                 raise NotImplementedError, 'unknown type given.'
               end

        base.new(response, attributes)
      end

      attr_reader :id, :age_limit, :caption, :favorite_id,
        :height, :is_linked, :is_manga, :publicity,
        :stats, :title, :tools, :width, :page_count,
        :book_style, :type, :metadata

      define_attribute_method :user, Response::User

      define_blob_method :image_urls

      define_time_method :created_time, :reuploaded_time

      def pages
        @attributes['metadata']['pages'].map do |base, memo|
          base['image_urls'].each_with_object({}) do |(key, value), memo|
            memo[key] = blob_image_url(value)
          end
        end
      rescue
        # XXX: workの種類によってはエラーが起こる？
        []
      end

      memoize(:pages)
    end
  end
end
